# ***************************************************************************
#
# BatterySensor - gDecklet sensor for laptop battery data.
#
# Author: Nicholas Rahn <nick at mobiledude dot com>
# Creation Date: 5 Sept. 2003
#
# Version: 0.2
#
# See README for history.
#
# ***************************************************************************

from sensor.Sensor import Sensor

import re
import os

#
# Battery sensor class for gDecklets.
#
# INPUTS: Current implementation works with:
#   APM 
#   ACPI (BAT0 or BAT1 [not both])
#
# OUTPUTS:
#   status
#   ac
#   percent
#   level
#   icon
#
class BatterySensor( Sensor ):

    RE_ACADAPTER = re.compile( "state:[\s]*on-line.*" )
    RE_BATPRESENT = re.compile( "present:[\s]*yes.*",
                               re.DOTALL );
    RE_CURRBATT = re.compile( ".*remaining capacity:[\s]*(?P<curr>[\d]+) .*",
                              re.DOTALL )
    RE_FULLBATT = re.compile( ".*last full capacity:[\s]*(?P<full>[\d]+) .*",
                              re.DOTALL )
    
    ACPI_BAT_BASE = "/proc/acpi/battery/"
    ACPI_INFO = ""
    ACPI_STATE = ""
    ACPI_AC_STATE = "/proc/acpi/ac_adapter/AC/state"
    
    APM_PROC = "/proc/apm"
    RE_APMPROC = re.compile( "(?P<junk1>[\d\.]+) (?P<junk2>[\d\.]+) (?P<apmflags>[\w]+) (?P<acstat>[\w]+) (?P<batstat>[\w]+) (?P<batflags>[\w]+) (?P<s1>[\w\?\-\%]+) (?P<s2>[\w\?\-\%]+) (?P<s3>[\w\?\-\%]+)" )
    
    AC_LEVEL = "ac"
    OK_LEVEL = "ok"
    WARN_LEVEL = "warn"
    ALERT_LEVEL = "alert"
    CRITICAL_LEVEL = "critical"

    OK_PERCENT = 100
    WARN_PERCENT = 40
    ALERT_PERCENT = 25
    CRITICAL_PERCENT = 15
    
    DEFAULT_DELAY = 2000
    
    def __init__( self ):

        # Instance variable for general battery status.
        self.__status = "AC"
        # Instance variable for AC status.
        self.__ac = "on-line"
        # Instance variable for battery percentage.
        self.__percent = self.OK_PERCENT
        # Instance variable for battery level.
        self.__color = self.AC_LEVEL
        # Instance variable for full battery level.
        self.__fullcapacity = 0
        # Instance variable for type of battery (ACPI, APM)
        self.__type = "acpi"
        
        # Call the constructor of the parent class.
        Sensor.__init__( self )

        # Auto detect the type of battery.
        if ( os.access( "/proc/acpi/battery/BAT0/info", os.F_OK ) ):
            self.__type = "acpi"

            batnum = self.getACPIBatNum()
            self.ACPI_INFO = self.ACPI_BAT_BASE + "BAT" + batnum + "/info"
            self.ACPI_STATE = self.ACPI_BAT_BASE + "BAT" + batnum + "/state"
        elif ( os.access( self.APM_PROC, os.F_OK ) ):
            self.__type = "apm"
        else:
            self.__type = "unknown"

        # Initialize the battery type.
        if ( self.__type == "acpi" ):
            self.initACPI()
        elif ( self.__type == "apm" ):
            self.initAPM()

    # Initialize APM.
    def initAPM( self ):
        self.__status = "not supported"
        self.__fullcapacity = 0

        # Run the timer once every 2 seconds.
        self._add_timer( self.DEFAULT_DELAY, self.setAPMBatteryOutput )
        return

    # Set the "output" variables for the sensor using APM battery.
    def setAPMBatteryOutput( self ):
        try:
            f = open( self.APM_PROC, "r" )
            data = f.read()
            f.close()
        except:
            # Can't get AC state! Probably not using APM.
            return 0
        
        self.updateAPMBatteryState( data )
        
        output = self._new_output()
        output.set("status", self.__status)
        output.set("ac", self.__ac)
        output.set("percent", self.__percent)
        output.set("level", self.__level)
        output.set("icon", self.__level + ".png")
        self._send_output( output )
        return 1

    # Update the member data with the current state of the APM battery.
    def updateAPMBatteryState( self, data ):
        #if ( self.RE_APMPROC.match( data ) ):
        acstat = self.RE_APMPROC.search( data ).group( "acstat" )
        s1 = self.RE_APMPROC.search( data ).group( "s1" )
            
        if ( acstat == "0x01" ):
            self.__ac = "on-line"
            self.__status = "AC"
            self.__percent = int(s1[0:(len(s1)-1)])
            self.__level = self.calcLevel( self.__percent )
        else:
            self.__ac = "off-line"
            self.__status = s1
            self.__percent = int(s1[0:(len(s1)-1)])
            self.__level = self.calcLevel( self.__percent )


    # Initialize ACPI.
    def initACPI( self ):
        # Get the full capacity of the battery.  We do this here to avoid the
        # performance hit we get every time we read /BATn/info.
        try:
            f = open( self.ACPI_INFO, "r" )
            data = f.read()
            self.__fullcapacity = float(self.RE_FULLBATT.
                                        search( data ).group( "full" ))
            f.close()
        except:
            # Can't get battery info! Probably not using ACPI.
            self.__fullcapacity = 0;

        # Run the timer once every 2 seconds.
        self._add_timer( self.DEFAULT_DELAY, self.setACPIBatteryOutput )


    # Get the ACPI battery number.
    def getACPIBatNum( self ):
        f = open( "/proc/acpi/battery/BAT0/info", "r" )
        data = f.read()
        f.close()
        if ( self.RE_BATPRESENT.match( data ) ):
            return "0"

        f = open( "/proc/acpi/battery/BAT1/info", "r" )
        data = f.read()
        f.close()
        if ( self.RE_BATPRESENT.match( data ) ):
            return "1"

        return "0"
    
    # Set the "output" variables for the sensor using ACPI battery.
    def setACPIBatteryOutput( self ):
        try:
            f = open( self.ACPI_AC_STATE, "r" )
            data = f.read()
            f.close()
        except:
            # Can't get AC state! Probably not using ACPI.
            return 0
        
        if ( self.RE_ACADAPTER.match( data ) ):
            self.__ac = "on-line"
            self.updateACPIBatteryState()
            self.__status = "AC"
        else:
            self.__ac = "off-line"
            self.updateACPIBatteryState()
            self.__status = str(self.__percent) + "%"
            
        output = self._new_output()
        output.set("status", self.__status)
        output.set("ac", self.__ac)
        output.set("percent", self.__percent)
        output.set("level", self.__level)
        output.set("icon", self.__level + ".png")
        self._send_output( output )
        return 1

    # Update the member data with the current state of the ACPI battery.
    def updateACPIBatteryState( self ):
        try:
            f = open( self.ACPI_STATE, "r" )
            data = f.read()
            current = self.RE_CURRBATT.search( data ).group( "curr" )
            f.close()
        except:
            # Can't get battery state! Probably not using ACPI!
            return

        # Set the percentage of battery left.
        self.__percent = int(int(current)/self.__fullcapacity * 100)
        # Correct for any differences between "design capacity", "last full
        # capacity" and "remaining capacity".
        if ( self.__percent > self.OK_PERCENT ):
            self.__percent = self.OK_PERCENT
        # Set the level code.
        self.__level = self.calcLevel( self.__percent )


    def calcLevel( self, per ):
        # Calculate the level code from the percent.
        if ( per == self.OK_PERCENT ):
            return self.AC_LEVEL
        elif ( per > self.WARN_PERCENT ):
            return self.OK_LEVEL
        elif ( per > self.ALERT_PERCENT ):
            return self.WARN_LEVEL
        elif ( per > self.CRITICAL_PERCENT ):
            return self.ALERT_LEVEL
        else:
            return self.CRITICAL_LEVEL
            
        

def new_sensor( args ): return apply( BatterySensor, args )
