# Copyright 2003 Jesse Andrews under GPL

from sensor.Sensor import Sensor
from utils.datatypes import *
import urllib

_bug_types = [ "UNCONFIRMED", "NEW", "ASSIGNED", "REOPENED", "NEEDINFO", "RESOLVED", "VERIFIED", "CLOSED" ]

class BugzillaSensor( Sensor ):

	def __init__( self ):
		Sensor.__init__( self )

		### QTY to show in the display (last qty) ###
		self._set_config_type( "qty", TYPE_INT, 3 )
		
		### product/bugzilla URL ###
		self._set_config_type( "product", TYPE_STRING, "gDesklets" )
		self._set_config_type( "bugzilla", TYPE_STRING, "http://bugzilla.gnome.org/buglist.cgi?" )

		### what bug types to worry about ###
		self._set_config_type( "UNCONFIRMED", TYPE_BOOL, 0)
		self._set_config_type( "NEW",         TYPE_BOOL, 1)
		self._set_config_type( "ASSIGNED",    TYPE_BOOL, 1)
		self._set_config_type( "REOPENED",    TYPE_BOOL, 1)
		self._set_config_type( "NEEDINFO",    TYPE_BOOL, 0)
		self._set_config_type( "RESOLVED",    TYPE_BOOL, 0)
		self._set_config_type( "VERIFIED",    TYPE_BOOL, 0)
		self._set_config_type( "CLOSED",      TYPE_BOOL, 0)
		
		self._add_timer( 0, self.__get_data )

	def constructQuery( self ):
		''' creates a query for the specified bugzilla with options given '''
		
		### all this data will be grabbed from the Sensor settings ###
		PRODUCT = self._get_config( 'product' )
		BUG_STATUS = ['UNCONFIRMED','NEW','ASSIGNED','REOPENED']
		BUGZILLA = self._get_config( 'bugzilla' )
		
		### this should be the same ###
		CMD = { 'cmdtype':'doit', 'order':'Bug Number', 'form_name':'query' }
	
		### start producing the URL ###
		query = BUGZILLA + urllib.urlencode( {'product':PRODUCT} )
		
		# UGLY HACK!
		# we have to do this since we repeat bug_status=TYPE1&bug_status=TYPE2
		# and urlencode works from dicts, which can only have one value per key
		query += ''.join(['&'+urllib.urlencode({'bug_status':bt}) for bt in _bug_types if self._get_config( bt )])
			
		# maybe I should just combine CMD & product, but this is the order it was created by
		# bugzilla, so I create it in this order
		query += "&"+urllib.urlencode( CMD )
		
		return query

	def getBugs( self ):
		''' 
		    returns a list of bugs
		    each bug is a dictionary containing the id, summary, ... 
		'''
		
		# we should really do something like caching/hashing/... so that if nothing has changed on the page,
		# don't reparse it...  maybe there is a way to get the time of the last change to bugzilla for a given "PRODUCT"
		# and only query if it has changed since the last time....  
		
		
		q = self.constructQuery()
		
		#### WE SHOULD ONLY DO THIS EVERY 10 minutes!!!! ####
		
		try:
			u = urllib.urlopen( q )
	
			# the real data is contained within a <TBODY> ... </TBODY> 
			data = u.read().split('TBODY>')[1]
	
			bugs = []
			# first two lines are blank, then a header we can ignore, then everything bug the last line are bugs!
			for line in data.split('\n')[2:-1]:
			
				bugdata = {}
				# the bug starts on the line after the first <TD> and then is seperated by <td ...>
				bugtxt = line.split('<TD>')[1].split('<td')
		
				# ugly parsing code that will break as soon as the HTML format changes!!!
				bugdata['id'] = (bugtxt[0].split('>')[1])[:-3]
				bugdata['severity'] = (bugtxt[1].split('nobr>')[1])[0:-2]
				bugdata['priority'] = (bugtxt[2].split('nobr>')[1])[0:-2]
				bugdata['owner'] = (bugtxt[3].split('nobr>')[1])[0:-2]
				bugdata['status'] = (bugtxt[4].split('nobr>')[1])[0:-2]
				bugdata['summary'] = (bugtxt[6].split('>')[1])[:-4]
			
				bugs.append( bugdata )
		except:
			bugs = []
		
		return bugs

	def __get_data( self ):
		
		output = self._new_output()
		
		bugs = self.getBugs()

		bugQty = len(bugs)
		if self._get_config( 'qty' ) == 0:
			dispQty = bugQty
		else:
			dispQty = min( bugQty, self._get_config( 'qty' ) )

		for i in range( dispQty ):
			output.set( "bugID[%d]"       % i, bugs[bugQty-i-1]['id']       )
			output.set( "bugSTATUS[%d]"   % i, bugs[bugQty-i-1]['status']   )
			output.set( "bugSUMMARY[%d]"  % i, bugs[bugQty-i-1]['summary']  )
			output.set( "bugSEVERITY[%d]" % i, bugs[bugQty-i-1]['severity'] )
			output.set( "bugPRIORITY[%d]" % i, bugs[bugQty-i-1]['priority'] )
			output.set( "bugOWNER[%d]"    % i, bugs[bugQty-i-1]['owner']    )

		output.set( "qty", dispQty  )

		output.set( "product", self._get_config( 'product')  )


		### Wait 10 minutes before the next query! ###
		self._add_timer( 10*60*1000, self.__get_data )
		self._send_output( output )

	def get_configurator( self ):
		conf = self._new_configurator()
		conf.set_name( "BugZilla" )
		conf.add_title( "We query the bugzilla every 10 minutes" )
		conf.add_title( "So, changes to configuration are not instantly shown" )
		
		conf.add_title( "Query" )
		conf.add_entry( "URL", "bugzilla", "Query CGI for bugzilla you want to check" )
		conf.add_entry( "Product", "product", "The product you want to query bugzilla about" )

		conf.add_title( "Bugs" )
		for bt in _bug_types:
			conf.add_checkbox( bt.capitalize(), bt, "If enabled, query for " + bt + " bugs" )

		conf.add_spin( "Bugs Shown", "qty", "How many of the most recent bugs you want shown (choose 0 for all)", 0, 50 )

		return conf


def new_sensor( args ): return apply( BugzillaSensor, args )


