
from utils.datatypes import *
from sensor.Sensor import Sensor

import os
import re

class CPUTemp (Sensor):
	"""
	Sensor that periodically checks /proc/acpi/thermal_zone/THRM/temperature to find the current CPU
	temperature.
		
	Configuration data:
		- low_temp			: If the CPU is at this temperature or below, no warning icon will be shown.
		- high_temp			: If the CPU is at this temperature or above, a critical warning will be shown.
		
	Output data:
		- temperature		: The current temperature.
		- icon				: The filename of the icon to display.
		- error				: Set to true if CPU temperature information could not be read.
	"""

	TEMPERATURE_STATUS="/proc/acpi/thermal_zone/THRM/temperature"

	def __init__(self, interval=1000):
		
		Sensor.__init__(self)		
		
		self._set_config_type("low_temp", TYPE_INT, 60)
		self._set_config_type("high_temp", TYPE_INT, 100)
		
		self._add_timer (int (interval), self.update_display)

	def get_configurator(self):
		configurator = self._new_configurator()

		configurator.set_name('CPU Temperature')

		configurator.add_title('CPU Temperature')
		configurator.add_spin ('Low temperature', 'low_temp', 'Temperature considered "low".', 0, 200)
		configurator.add_spin ('High temperature', 'high_temp', 'Temperature considered critical.', 0, 200)
		
		return configurator


	def update_display (self):
	
		(temp, error) = self.get_temperature ()
	
		data = self._new_output()

		icon = "gfx/error.png"
			
		if not error:
			low_temp = int (self._get_config ("low_temp"))
			high_temp = int (self._get_config ("high_temp"))
			
			if temp <= low_temp:
				icon = "gfx/low.png"
			elif temp >= high_temp:
				icon = "gfx/high.png"
			else:
				icon = "gfx/medium.png"
			
		data.set ("temperature", "%d°C" % (temp))
		data.set ("icon", icon)
		data.set ("error", error)
		
		self._send_output(data)

		return 1

	def get_temperature (self):
		"""
		Read the current CPU temperature form /proc/acpi/thermal_zone/THRM/temperature
		"""
		
		try:
			status_file = open (self.TEMPERATURE_STATUS, "r")
		except:
			return (0, 1)
		
		temp = 0
		error = 1
		
		try:
			status = status_file.read ()
			match = re.search ("(\d+) C", status)
			if match:
				temp = int (match.group (1))
				error = 0
		finally:
			status_file.close ()
		
		return (temp, error)
				
def new_sensor(args):
        return apply(CPUTemp, args)
