import os
import re

'''This is an abstract class. Platform dependent code should subclass it'''
class DiskIoSensor:
    '''Returns (read, write) bytes since boot up'''
    def get_read_write_bytes(self):
        raise Exception, "Please implement get_read_write_bytes()"

'''Linux IO sensor, 2.4 reads from /proc/stat, 2.6 reads from /proc/diskstats'''
class LinuxDiskIoSensor(DiskIoSensor):

    PROC_DISKSTATS = "/proc/diskstats"
    PROC_STAT = "/proc/stat"
    PROC_STAT_REGEX = re.compile("\(\d+,\d+\):\(\d+,\d+,(\d+),\d+,(\d+)\)")

    '''Constructor needs a kernel version'''
    def __init__(self, kernel_version):
        self.__kernel_version = kernel_version

    '''@see IoSensor.get_read_write_bytes()'''
    def get_read_write_bytes(self):
        if self.__kernel_version.find("2.4") >= 0:
            return self.__parse_proc_stat()
        elif self.__kernel_version.find("2.6") >= 0:
            return self.__parse_proc_diskstats()

    '''returns lines from stats file'''
    def __read_stats_file(self, filename):
        stats_file = open(filename, "r")
        lines = stats_file.readlines()
        stats_file.close()
        return lines
        
    '''Parse /proc/stat.
    Format disk_io: (major,minor):(total,read_count,read_sectors,write_count,write_sectors) ...
    Each sector is 512 bytes.'''
    def __parse_proc_stat(self):
        read = 0
        write = 0
        lines = self.__read_stats_file(LinuxDiskIoSensor.PROC_STAT)
        for line in lines:
            if line.startswith("disk_io: "):
                matches = LinuxDiskIoSensor.PROC_STAT_REGEX.findall(line)
                read = sum([long(x[0]) for x in matches])
                write = sum([long(x[1]) for x in matches])
        read = long(read) << 9
        write = long(write) << 9
        return (read, write)

    '''Parse /proc/diskstats.
    Format in /usr/src/linux/Documentation/iostats.txt.'''
    def __parse_proc_diskstats(self):
        read = 0
        write = 0
        lines = self.__read_stats_file(LinuxDiskIoSensor.PROC_DISKSTATS)
        for line in lines:
            values = line.split()
            if len(values) == 14:
                read = read + long(values[5])
                write = write + long(values[9])
        read = long(read) << 9      # << 9 = * (2**9)
        write = long(write) << 9    # 2**9 = 512
        return (read, write)

'''This is the Factory to create platform dependent sensors (not Sensor class)'''
class DiskIoSensorFactory:

    def createSensor():
        kernel_name, kernel_version = os.popen2("uname -sr")[1].readline().strip().split()
        if kernel_name == "Linux":
            return LinuxDiskIoSensor(kernel_version)
    createSensor = staticmethod(createSensor)