from sensor.Sensor import Sensor
from utils.datatypes import *

import ossaudiodev

class Mixer(Sensor):
	"""
	Sensor that finds the volume of a mixer device, and can be used to set the volume.
	
	by Martin Aspeli <optilude@gmx.net>
	
	Configuration data:
	
	  - device    : The mixer device (e.g. /dev/mixer)
	  - channel   : The number of the channel to control
	  - increment : How many percent up/down the scroll wheel changes the volume
	
	Output data:
	
	  - volume    : The volume as an integer from 0-100
	  - device    : The name of the mixing device
	  - channel   : The name of the channel
	  
	"""
		
	# These come from /usr/include/linux/soundcards.h
	CHANNELS = { "Volume"        : ossaudiodev.SOUND_MIXER_VOLUME,
				 "Bass"          : ossaudiodev.SOUND_MIXER_BASS,
				 "Treble"        : ossaudiodev.SOUND_MIXER_BASS,
				 "Synth"         : ossaudiodev.SOUND_MIXER_SYNTH,
	             "PCM"           : ossaudiodev.SOUND_MIXER_PCM,
	             "PC speaker"    : ossaudiodev.SOUND_MIXER_SPEAKER,
	             "Line out"      : ossaudiodev.SOUND_MIXER_LINE,
	             "Microphone"    : ossaudiodev.SOUND_MIXER_MIC,
	             "CD audio"      : ossaudiodev.SOUND_MIXER_CD,
	             "Input gain"    : ossaudiodev.SOUND_MIXER_IGAIN,
				 "Output gain"   : ossaudiodev.SOUND_MIXER_OGAIN,
				 "Line 1"        : ossaudiodev.SOUND_MIXER_LINE1,
				 "Line 2"        : ossaudiodev.SOUND_MIXER_LINE2,
				 "Line 3"        : ossaudiodev.SOUND_MIXER_LINE3,
				 "Digital 1"     : ossaudiodev.SOUND_MIXER_DIGITAL1,
				 "Digital 2"     : ossaudiodev.SOUND_MIXER_DIGITAL2,
				 "Digital 3"     : ossaudiodev.SOUND_MIXER_DIGITAL3,
				 "Phone in"      : ossaudiodev.SOUND_MIXER_PHONEIN,
				 "Phone out"     : ossaudiodev.SOUND_MIXER_PHONEOUT,
				 "Video"         : ossaudiodev.SOUND_MIXER_VIDEO,
				 "Radio"         : ossaudiodev.SOUND_MIXER_RADIO,
				 "Monitor"       : ossaudiodev.SOUND_MIXER_MONITOR }
	
	def __init__(self, width, increment = 5, device = "/dev/mixer", 
                       channel = "Volume", interval = 100):
		"""
		Initialise:
			width     - The width in pixels of the gauge the sensor is controlling. This is 
                         necessary to make clicking to set the volume work.
            increment - How many percent up/down the scroll wheel changes the volume
            device    - The initial mixing device (can be set in configurator)
            channel   - The channel (See CHANNELS array) to control
            interval  - The update interval (affects responsiveness to click/scroll)
		"""
		Sensor.__init__(self)

		# The x position of the cursor is used to figure out how much to set the volume to; 		# The volume will be set to (xpos * 100) / width
		self.__xpos = 0
		self.__width = int (width)

		channel_number = self.CHANNELS["Volume"]
		if channel in self.CHANNELS:
			channel_number = self.CHANNELS[channel]

		self._set_config_type ("device", TYPE_STRING, "/dev/mixer")
		self._set_config_type ("channel", TYPE_INT, channel_number)
		self._set_config_type ("increment", TYPE_INT, int (increment))

		self._add_timer(int (interval), self._on_tick)

	def get_configurator (self):
		configurator = self._new_configurator ()

		configurator.set_name ("Mixer")

		configurator.add_title ("Audio Mixer")
		configurator.add_entry ("Device:", "device", "Mixer device (normally /dev/mixer)")
		configurator.add_option ("Channel:", "channel", 
								"Channel to control", self.CHANNELS.items ())
		configurator.add_spin ("Wheel increment:", "increment", 
			"How many percent to increase/decrease the volume by when using the scroll wheel", 
			1, 10)
			
		return configurator


	def _on_tick(self):

		data = self._new_output()
		volume = self._get_volume ()
		
		if volume >= 0:
			data.set ("device", self._get_config ("device"))
			data.set ("channel", self._get_channel_name (self._get_config ("channel")))
			data.set ("volume", volume)
		else:
			data.set ("device", "Error: Could not open mixer")
			data.set ("channel", "Error: Could not open mixer")
			data.set ("volume", -1)

		self._send_output (data)
		return 1

	def call_action(self, action, path, args = []):
		if action == "move":
			self.__xpos = args[0]
		elif action == "click":	
			volume = (self.__xpos * 100) / self.__width 
			self._set_volume (volume)
		elif action == "scroll":

			increment = 0
			
			if args[0] == 0:   # Volume up
				increment = self._get_config ("increment")
			elif args[0] == 1: # Volume down
				increment = self._get_config ("increment") * -1
				
			if increment != 0:
				self._change_volume (increment)

	def _get_channel_name (self, channel):
		"Get the name of the given channel"
		
		for k, v in self.CHANNELS.items ():
			if v == channel:
				return k
			
		return "Volume"		

	def _get_mixer (self):
		"Open a mixer object and return it, or None if an error occurred device was invalid"
		
		device = self._get_config ("device")
		mixer = None
		
		try:
			mixer = ossaudiodev.openmixer (device)
			if not mixer:
				raise "could not open mixer"
		except:
			return None

		return mixer
		
	def _get_volume (self):
		"""
		Get the volume of the mixer and channel set in options. Return volume on success, -1  
        if device could not be opened or channel not found.
		"""
		
		mixer = None
		channel = None

		try:
			mixer = self._get_mixer ()
			channel = self._get_config ("channel")
			if mixer and channel >= 0:
				volumes = mixer.get (channel)
				volume = (volumes[0] + volumes[1])/2
				return volume
			else:
				return -1
		finally:
			if mixer:
				mixer.close ()
	

	def _set_volume (self, volume):
		"Set the volume to the given value (0-100)"
	
		mixer = None
		channel = None

		try:
			mixer = self._get_mixer ()
			channel = self._get_config ("channel")
		
			if mixer and channel >= 0:
				mixer.set(channel, (volume, volume))
		finally:
			if mixer:
				mixer.close ()

	def _change_volume (self, increment):
		"Change the volume by increment (may be negative), but stay within range 0-100"
		
		mixer = None
		channel = None

		try:
			mixer = self._get_mixer ()
			channel = self._get_config ("channel")
	
			if mixer and channel >= 0:
				volumes = mixer.get (channel)
				volume = (volumes[0] + volumes[1])/2

				new_volume = volume + increment
				if new_volume < 0:
					new_volume = 0
				elif new_volume > 100:
					new_volume = 100
				
				mixer.set(channel, (new_volume, new_volume))
		finally:
			if mixer:
				mixer.close ()		

def new_sensor (args): return apply (Mixer, args)
