from ConfigParser import ConfigParser
import os
import commands


#
# Class for looking up icons using icon themes. This comprehensive
# implementation follows the specification of freedesktop.org.
# Due to subtle differences between GNOME and KDE, this only works on GNOME at
# the moment.
#
# For details of the spec, see:
#   http://www.freedesktop.org/standards/icon-theme-spec/
#
#
# Copyright (C) 2003 Martin Grimme
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
class IconTheme:

    # the default icon theme
    __DEFAULT_THEME = "gnome"

    # the valid extensions
    __EXTENSIONS = ("png", "svg", "xpm")


    # what we can run to get the prefix
    __PREFIX = "DIR=`which gnome-desktop-item-edit` && echo `dirname $DIR`/.."

    # the name of the theme index file
    __INDEX_FILE = "index.theme"

    # scaling types of icons
    __TYPE_FIXED = "Fixed"
    __TYPE_SCALED = "Scaled"
    __TYPE_THRESHOLD = "Threshold"
    

    # record for storing theme directory settings
    class __ThemeDir:
        def __init__(self):
            self.dir = ""
            self.size = 48
            self.context = ""
            self.type = ""
            self.minsize = 48
            self.maxsize = 48
            self.threshold = 2
            


    #
    # Constructor.
    #
    def __init__(self, themename = ""):

        self.__basedirs = []
        self.__subdirs = []

        self.__file_cache = {}

        self.load_theme(themename or self.__DEFAULT_THEME)



    #
    # Loads the given theme.
    #
    def load_theme(self, themename):

        self.__basedirs = self.__get_base_directories()
        self.__subdirs = self.__get_theme_subdirs(themename)



    #
    # Checks whether a file exists.
    #
    def __exists(self, file):

        if (not self.__file_cache.has_key(file)):
            self.__file_cache[file] = os.path.exists(file)

        return self.__file_cache[file]



    #
    # Reads the given theme index file and returns a list of ThemeDir records
    # and the name of the inherited theme.
    #
    def __read_index_file(self, themename, indexfile):

        def get(parser, d, option, default):
            if (parser.has_option(d, option)):
                return parser.get(d, option)
            else:
                return default

        parser = ConfigParser()
        parser.read(indexfile)
        inherits = get(parser, "Icon Theme", "Inherits", "").split(",")
        dirs = get(parser, "Icon Theme", "Directories", "").split(",")

        subdirs = []
        for d in dirs:
            entry = self.__ThemeDir()
            entry.dir = os.path.join(themename, d)
            entry.type = get(parser, d, "Type", self.__TYPE_THRESHOLD)
            size = get(parser, d, "Size", "48")
            if (size == "Scalable"): entry.size = 48
            else: entry.size = int(size)
            entry.context = get(parser, d, "Context", "")
            entry.maxsize = int(get(parser, d, "MinSize", entry.size))
            entry.minsize = int(get(parser, d, "MaxSize", entry.size))
            entry.threshold = int(get(parser, d, "Threshold", "2"))
            subdirs.append(entry)
        #end for

        return (subdirs, inherits)
                           
        

    #
    # Returns the base directories where we have to look for themes.
    #
    def __get_base_directories(self):

        fail, prefix = \
           commands.getstatusoutput(self.__PREFIX)
        if (fail): prefix = "/usr"

        path = os.path.join(prefix, "share")


        dirs = [os.path.join(os.path.expanduser("~"), ".icons"),
                os.path.join(path, "icons"),
                os.path.join(path, "pixmaps")]

        return dirs



    #
    # Returns the list of all icon directories where we have to look for icons.
    # This recursively includes the directories of inherited themes.
    #
    def __get_theme_subdirs(self, themename):

        dirs = []
        for dir in self.__basedirs:
            filename = os.path.join(dir, themename, self.__INDEX_FILE)
            if (os.path.exists(filename)):
                dirs, inherits = self.__read_index_file(themename, filename)
                if (inherits):
                    for i in inherits:
                        # don't allow endless recursions
                        if (i != themename):
                            dirs += self.__get_theme_subdirs(i)
                #end if
                
            #end if
        #end for

        return dirs



    #
    # Returns whether a directory matches the given icon size.
    #
    def __directory_matches_size(self, subdir, iconsize):

        if (subdir.type == self.__TYPE_FIXED):
            return (subdir.size == iconsize)
        elif (subdir.type == self.__TYPE_SCALED):
            return (subdir.minsize <= iconsize <= subdir.maxsize)
        elif (subdir.type == self.__TYPE_THRESHOLD):
            return (subdir.size - subdir.threshold <= iconsize <=
                    subdir.size + subdir.threshold)




    #
    # Returns the size difference between a directory and the given icon size.
    #
    def __directory_size_distance(self, subdir, iconsize):

        if (subdir.type == self.__TYPE_FIXED):
            return abs(subdir.size - iconsize)

        elif (subdir.type == self.__TYPE_SCALED):
            if (iconsize < subdir.minsize):
                return subdir.minsize - iconsize
            elif (iconsize > subdir.maxsize):
                return iconsize - subdir.maxsize
            else:
                return 0

        elif (subdir.type == self.__TYPE_THRESHOLD):
            if (iconsize < subdir.size - subdir.threshold):
                return subdir.minsize - iconsize
            elif (iconsize > subdir.size + subdir.threshold):
                return iconsize - subdir.maxsize
            else:
                return 0

        else:
            return 0
        



    #
    # Return the absolute path for the given icon name and, optionally, the
    # given icon size and maybe the desired context.
    # Returns an empty string if the icon could not be found.
    #
    def lookup_icon(self, iconname, size = 48, context = ""):

        # don't search for icons with absolute paths
        if (iconname and iconname[0] == "/"): return iconname

        # disregard extension
        iconname, ext = os.path.splitext(iconname)

        # look for exact match
        for subdir in self.__subdirs:
            if (context and not context == subdir.context): continue
            for directory in self.__basedirs:
                for extension in self.__EXTENSIONS:
                    if (self.__directory_matches_size(subdir, size)):
                        filename1 = os.path.join(directory,
                                                subdir.dir,
                                                iconname)
                        filename2 = os.path.join(directory,
                                                subdir.dir,
                                                iconname + "." + extension)

                        if (self.__exists(filename1)):
                            return filename1
                        elif (self.__exists(filename2)):
                            return filename2
                    #end if
                #end for
            #end for
        #end for


        # look for closest match
        minimal_size = 0xffffff
        closest_filename = ""
        for subdir in self.__subdirs:
            if (context and not context == subdir.context): continue
            for directory in self.__basedirs:
                for extension in self.__EXTENSIONS:
                    filename1 = os.path.join(directory,
                                            subdir.dir,
                                            iconname)
                    filename2 = os.path.join(directory,
                                            subdir.dir,
                                            iconname + "." + extension)

                    if (self.__exists(filename1) and
                        self.__directory_size_distance(subdir, size) <
                        minimal_size):
                        closest_filename = filename1
                        minimal_size = self.__directory_size_distance(subdir,
                                                                      size)
                    elif (self.__exists(filename2) and
                        self.__directory_size_distance(subdir, size) <
                        minimal_size):
                        closest_filename = filename2
                        minimal_size = self.__directory_size_distance(subdir,
                                                                      size)
                    #end if

                #end for
            #end for
        #end for

        if (closest_filename): return closest_filename


        # look for unthemed icons
        for directory in self.__basedirs:
            for extension in self.__EXTENSIONS:
                filename1 = os.path.join(directory, iconname)
                filename2 = os.path.join(directory, iconname + "." + extension)
                if (self.__exists(filename1)): return filename1
                elif (self.__exists(filename2)): return filename2
            #end for
        #end for


        return ""




if (__name__ == "__main__"):
    import sys

    try:
        theme = sys.argv[1]
        name = sys.argv[2]
    except:
        print "Usage: python " + sys.argv[0] + " <theme> <icon name>\n"
        sys.exit(1)
        
    t = IconTheme(theme)
    print t.lookup_icon(name)
