from sensor.Sensor import Sensor
from IconSet import IconSet
from Animator import Animator
from utils.datatypes import *
from utils import i18n

import os
import commands


#
# Iconbar for starters (.desktop files).
#
class StarterBar(Sensor):

    # panel orientation values
    TOP = "top"
    BOTTOM = "bottom"
    LEFT = "left"
    RIGHT = "right"


    def __init__(self):

        global _; _ = i18n.Translator("starterbar-sensor")

        # the set of icons
        self.__icon_set = None

        # the animator
        self.__animator = Animator()

        # the index of the currently selected icon or -1
        self.__current_index = -1

        # the index of the currently dragged icon or -1 if none is dragged
        self.__dragged_icon = -1

        # the current orientation of the panel
        self.__orientation = self.BOTTOM

        # the jump align for icons
        self.__jump_align = -1

        # the scaling of the panel
        self.__panel_scaling = 0.5

        # the size of the panel
        self.__panel_size = 0

        # the center of the panel (used for distance calculations)
        self.__panel_center = 0
        

        Sensor.__init__(self)
        self._set_config_type("orientation", TYPE_STRING, self.__orientation)
        self._set_config_type("animation", TYPE_BOOL, 1)
        self._set_config_type("stretch", TYPE_BOOL, 1)
        self._set_config_type("hilight", TYPE_BOOL, 1)
        self._set_config_type("panel-scaling", TYPE_INT, 100)
        self._set_config_type("background", TYPE_STRING, "gfx/bg.png")
        self._set_config_type("foreground", TYPE_STRING, "")
        self._watch_config(self.__on_watch_config)

        self._add_timer(0, self.__initialize)




    def __initialize(self):

        self.__icon_set = IconSet(self._get_config_id(), self._get_path())
        self._set_paths_to_purge([self.__icon_set.get_icon_path()])
        self.__icon_set.set_visualizer(self.__on_visualize)
        
        self._add_timer(0, self.__icon_set.load_icons)
        self._add_timer(50, self.__animator.animation, self.__on_animate)

        self.__panel_scaling = self._get_config("panel-scaling") / 100.0
        self.__setup_panel()




    #
    # Visualizes the panel.
    #
    def __on_visualize(self, src):

        self.__set_orientation(self.__get_orientation())
        self.__setup_panel()



    #
    # Visualizes animations.
    #
    def __on_animate(self, zoom, align, jump):

        output = self._new_output()
        for i in xrange(len(zoom)):                
            output.set("icon-scale[%(i)d]" % vars(),
                       zoom[i] * self.__panel_scaling)
            if (self.__orientation in [self.TOP, self.BOTTOM]):
                output.set("icon-align-x[%(i)d]" % vars(), 0.5 + align[i])
                output.set("icon-align-y[%(i)d]" % vars(),
                           abs(self.__jump_align + jump[i]))
            elif (self.__orientation in [self.LEFT, self.RIGHT]):
                output.set("icon-align-y[%(i)d]" % vars(), 0.5 + align[i])
                output.set("icon-align-x[%(i)d]" % vars(),
                           abs(self.__jump_align + jump[i]))

        #end for
        self._send_output(output)



    #
    # Hilights the icon with the given index.
    #
    def __hilight_icon(self, index):

        length = self.__icon_set.get_length()
        output = self._new_output()
        for i in xrange(length):
            if (i == index):
                output.set("saturation[%(i)d]" % vars(), 4.0)
            else:
                output.set("saturation[%(i)d]" % vars(), 1.0)
        #end for

        self._send_output(output)
        


    #
    # Reacts on changes in the configuration.
    #
    def __on_watch_config(self, key, value):

        if (key == "background"):
            output = self._new_output()
            output.set("background", value)
            self._send_output(output)

        elif (key == "foreground"):
            output = self._new_output()
            output.set("foreground", value)
            self._send_output(output)

        elif (key == "orientation"):
            self.__setup_panel()

        elif (key == "panel-scaling"):
            self.__panel_scaling = float(value) / 100.0
            self.__setup_panel()
        


    #
    # Sets up the panel.
    #
    def __setup_panel(self):

        self.__set_orientation(self._get_config("orientation"))

        length = self.__icon_set.get_length()
        output = self._new_output()
        for i in xrange(length):
            icon = self.__icon_set.get_icon(i)
            output.set("icon-uri[%(i)d]" % vars(), icon.get_pixmap())
            #output.set("icon-scale[%(i)d]" % vars(), self.__panel_scaling)

            if (self.__orientation in [self.TOP, self.BOTTOM]):
                output.set("icon-align-x[%(i)d]" % vars(), 0.5)
                output.set("icon-align-y[%(i)d]" % vars(),
                           abs(self.__jump_align))
            elif (self.__orientation in [self.LEFT, self.RIGHT]):
                output.set("icon-align-y[%(i)d]" % vars(), 0.5)
                output.set("icon-align-x[%(i)d]" % vars(),
                           abs(self.__jump_align))
        #end for

        output.set("length", length)
        output.set("background", self._get_config("background"))
        output.set("foreground", self._get_config("foreground"))
        self._send_output(output)
        self.__animator.set_size(length)



    #
    # Sets the orientation of the panel.
    #
    def __set_orientation(self, orientation):

        length = self.__icon_set.get_length()
        
        distance = int(56 * self.__panel_scaling)
        size = max(32, length * 56)
        totalsize = (length - 1) * 56 + 120
        
        output = self._new_output()
        if (orientation == self.TOP):
            layout = "horizontal, %(distance)d" % vars()
            panel = (36, 0, size, 50, totalsize, 86)
            align_width = 120
            align_height = 86
            center = 0
            self.__jump_align = 0
        elif (orientation == self.BOTTOM):
            layout = "horizontal, %(distance)d" % vars()
            panel = (36, 36, size, 50, totalsize, 86)
            align_width = 120
            align_height = 86
            center = 86
            self.__jump_align = -1
        elif (orientation == self.LEFT):
            layout = "vertical, %(distance)d" % vars()
            panel = (0, 36, 50, size, 86, totalsize)
            align_width = 86
            align_height = 120
            center = 0
            self.__jump_align = 0
        elif (orientation == self.RIGHT):
            layout = "vertical, %(distance)d" % vars()
            panel = (36, 36, 50, size, 86, totalsize)
            align_width = 86
            align_height = 120
            center = 86
            self.__jump_align = -1

        scale = self.__panel_scaling
        output.set("layout", layout)
        output.set("panel-x", panel[0] * scale)
        output.set("panel-y", panel[1] * scale)
        output.set("panel-width", panel[2] * scale - 8)
        output.set("panel-height", panel[3] * scale - 8)
        for i in xrange(self.__icon_set.get_length()):
            output.set("align-width[" + str(i) + "]", align_width * scale)
            output.set("align-height[" + str(i) + "]", align_height * scale)

        output.set("total-width", panel[4] * scale + 10)
        output.set("total-height", panel[5] * scale + 10)
        
        self._send_output(output)
        self.__orientation = orientation
        self.__panel_size = (size + 2 * max(panel[0], panel[1])) * scale
        self.__panel_center = center * scale



    #
    # Stretches/unstretches the panel.
    #
    def __set_stretched(self, stretched):

        orientation = self._get_config("orientation")
        stretch = stretched and 36 or 0
        
        length = self.__icon_set.get_length()
        distance = int(56 * self.__panel_scaling)
        size = max(32, length * 56)
        
        output = self._new_output()
        if (orientation == self.TOP):
            panel = (36 - stretch, 0, size + 2 * stretch, 50)
        elif (orientation == self.BOTTOM):
            panel = (36 - stretch, 36, size + 2 * stretch, 50)
        elif (orientation == self.LEFT):
            panel = (0, 36 - stretch, 50, size + 2 * stretch)
        elif (orientation == self.RIGHT):
            panel = (36, 36 - stretch, 50, size + 2 * stretch)

        scale = self.__panel_scaling
        output.set("panel-x", panel[0] * scale)
        output.set("panel-y", panel[1] * scale)
        output.set("panel-width", panel[2] * scale - 8)
        output.set("panel-height", panel[3] * scale - 8)                
        self._send_output(output)

        


    #
    # Returns the orientation of the panel.
    #
    def __get_orientation(self):

        return self.__orientation



    #
    # Reacts on user actions.
    #
    def call_action(self, action, path, args = []):

        # mouse motion
        if (action == "motion"):
            x = int(args[0])
            y = int(args[1])

            orientation = self.__get_orientation()
            if (orientation in [self.TOP, self.BOTTOM]):
                value = x
                distance = abs(self.__panel_center - y)
            else:
                value = y
                distance = abs(self.__panel_center - x)

            damping = max(0, (-distance / self.__panel_scaling) / 160.0 + 1.0)
            # make value discrete (saves CPU power)
            damping = int(damping * 20) / 20.0

            length = self.__icon_set.get_length()
            pos = (value / float(self.__panel_size)) * length
            if (self.__dragged_icon == -1):
                if (self._get_config("animation")):
                    self.__animator.hilight(pos - 0.5, damping)

            # move dragged icon
            elif (self.__dragged_icon != -1 and
                  self.__current_index != self.__dragged_icon):
                self.__icon_set.swap_icons(self.__dragged_icon,
                                           self.__current_index)
                self.__dragged_icon = self.__current_index



        # enter icon
        elif (action == "enter-icon"):
            index = path[-1]
            self.__current_index = index
            if (self._get_config("hilight")): self.__hilight_icon(index)


        # enter icon panel
        elif (action == "enter"):
            if (self._get_config("animation") and self._get_config("stretch")):
                self.__set_stretched(1)
            
        # leave icon panel
        elif (action == "leave"):
            if (self._get_config("stretch")): self.__set_stretched(0)
            self.__current_index = -1
            self.__animator.unhilight()
            self.__hilight_icon(-1)
            self.__dragged_icon = -1

        # start application
        elif (action == "launch"):
            index = self.__current_index
            if (self.__dragged_icon == -1 or self.__dragged_icon == index):
                self.__launch_icon(index)
            self.__dragged_icon = -1

        # drag icon
        elif (action == "drag-icon"):
            index = self.__current_index
            self.__dragged_icon = index
            self.__animator.unhilight()

        # drop dragged icon
        elif (action == "drop-icon"):
            index = int(path[-1])
            if (index != self.__dragged_icon):
                #self.__icon_set.move_icon(self.__dragged_icon, index)
                #self.__icon_set.swap_icons(index, self.__dragged_icon)
                pass
            self.__dragged_icon = -1
            
        # open menu
        elif (action == "menu"):
            removable = (self.__current_index != -1)

            menu = [("New starter...", 1, None, self.__add_starter, []),
                    ("Edit starter...", removable, None, self.__edit_starter,
                     [self.__current_index]),
                    ("Remove starter", removable, None, self.__remove_starter,
                     [self.__current_index])
                    ]
            self._open_menu(menu)

        # add icon
        elif (action == "add"):
            for file in args[0]:
                self.__icon_set.add_icon(file)

        # launch with file as argument
        elif (action == "launch-with"):
            index = path[-1]
            self.__current_index = index
            for file in args[0]:
                self.__launch_icon(index, file)

        elif (action == "scale-bar"):
            direction = args[0]
            scaling = self._get_config("panel-scaling")
            if (direction == 0):
                scaling -= 10
            else:
                scaling += 10
            scaling = max(20, min(500, scaling))
            self._set_config("panel-scaling", scaling)


    #
    # Launches the given icon with the given file as argument.
    #
    def __launch_icon(self, index, file = ""):

        cmd = self.__icon_set.get_icon(index).get_command()
        cmd = cmd.replace("%U", file)

        os.system("cd ~ && " + cmd + " & disown")
        self.__animator.unhilight()
        if (self._get_config("animation")):
            self.__animator.jump(index)


    #
    # Adds a new starter.
    #
    def __add_starter(self, src, *args):

        self.__icon_set.add_icon()



    #
    # Removes the given starter.
    #
    def __remove_starter(self, src, index):

        if (index == -1): return
        self.__icon_set.remove_icon(index)



    #
    # Edits the given starter.
    #
    def __edit_starter(self, src, index):

        if (index == -1): return
        self.__icon_set.edit_icon(index)



    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name(_("Panel"))
        configurator.add_title(_("Appearance"))
        configurator.add_option(_("Orientation:"), "orientation",
                                _("The orientation of the panel"),
                                [(_("top"), self.TOP),
                                 (_("bottom"), self.BOTTOM),
                                 (_("left"), self.LEFT),
                                 (_("right"), self.RIGHT)]
                                )
        configurator.add_spin(_("Zoom:"), "panel-scaling",
                              _("The size of the panel in percents"),
                              10, 500)
        configurator.add_checkbox(_("Show animations"), "animation",
                                  _("Toggles eye candy animations on or off"))
        configurator.add_checkbox(_("Stretch panel (only when animated)"),
                                  "stretch",
                                  _("Toggles stretching of the panel on "
                                    "or off"))
        configurator.add_checkbox(_("Hilight icons"), "hilight",
                                  _("Toggles highlighting the selected icon "
                                    "on or off"))
        configurator.add_file_selector(_("Background:"), "background",
                                       _("Selects the background image file."))
        configurator.add_file_selector(_("Foreground:"), "foreground",
                                       _("Selects the foreground image file."))

        return configurator



    #
    # Executes shutdown tasks.
    #
    def _shutdown(self):

        #self.__icon_set.close()
        pass



def new_sensor(args): return apply(StarterBar, args)
