# vim: expandtab : tabstop=4 : shiftwidth=4 : smarttab
# coding: UTF-8
# Version 0.6
# - parse output of 'sensors' instead of reading /proc/... directly
# Version 0.5
# Version 0.4
# Version 0.3
# Version 0.2
# Version 0.1
# Originally written by Elias Pschernig <elias@users.sf.net>.
# Originally adapted from code posted at
# http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=207315.
# Originally based on the hddtemp sensor by Sebastien Bacher.

# command to parse
command = "sensors -A"
# downscaling of values for plotters
sens_RPM = 2    # fans
sens_V = 0.005  # voltages
sens_T = 0.1    # temperatures

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import sys
import re
import commands

class LMSensorsSensor(Sensor):

    """Sensor for retrieving lm-sensors information."""

    def __init__(self):
        """Sets up the sensor."""
        global _; _ = i18n.Translator("lmsensors-sensor")
        Sensor.__init__(self)
        self.__sensorlist = []
        self.__active = 0
        self._set_config_type("update", TYPE_INT, 10)

        # It's not allowed to send output from inside this function!
        # It's also not allowed to send output from another thread(!), so we
        # need a ticker.
        self._add_timer(1000, self.__tick)
        self._add_thread(self.__get_value_thread)

    def get_configurator(self):
        """Called everytime the config menu is selected."""
        configurator = self._new_configurator()
        configurator.set_name(_("LMSensors"))
        if self.__active:
            configurator.add_spin(_("Update every (seconds):"), "update",
                _("Updates the sensor value every x seconds"), 1, 900)
            if len (self.__sensorlist):
                configurator.add_title(_("Available sensors:"))
                for sensor in self.__sensorlist:
                    configurator.add_checkbox (sensor["name"], sensor["name"],
                    _("Toggle this sensor"))
            else:
                 configurator.add_title(_("No sensors were found."))
                 configurator.add_title(_("Is lmsensors installed and working?"))
        else:
            configurator.add_title(
                _("Not done reading hardware sensors yet..."))

        return configurator
  
    def __get_sensor_data (self, append = False):
        """Runs the sensor program and parses the output."""
        (status, output) = commands.getstatusoutput (command)

        if status != 0: return

        lines = output.splitlines()
 
        for line in lines: 
            # line with just one single word on it -> chip name
            if re.match ("^[^:\s]+$", line):
                chipname = line
            else:
                # name colon number open-parenthesis
                # TODO: also extract max and min, and color accordingly
                val = re.match ("^\s*([^:]+?)\s*:\s*([^\(]+?)\s*\(", line)
                if val:
                    val = val.groups()
                    if val[0] != "alarms":
                        sensor = {}
                        sensor["name"] = re.sub ("[^\w\d\/-]", "_", chipname + "/" + val[0])
                        sensor["value"] = re.sub ("\xb0", "°", val[1])
                        if append:
                            # new entry
                            sensor["label"] = val[0]
                            sensor["enabled"] = False
                            self.__sensorlist.append (sensor)
                        else:
                            # update value
                            for s in self.__sensorlist:
                                if s["name"] == sensor["name"]:
                                    s["value"] = sensor["value"] 
        self.updated = True
        return

    def __read_sensors (self):
        """Reads the sensor values."""
        self.__get_sensor_data ()
        for sensor in self.__sensorlist:
            if sensor["enabled"]:
                is_RPM = sensor["value"].find ("RPM") != -1
                is_V = sensor["value"].find ("V") != -1
                if is_RPM:
                    sensibility = sens_RPM
                elif is_V:
                    sensibility = sens_V
                else:
                    sensibility = sens_T                 
                v = float (re.sub ("[^\d]+$", "", sensor["value"]))
                if "last" in sensor:
                    sensor["plot"] += (v - sensor["last"]) / sensibility
                else:
                    sensor["plot"] = 50
                sensor["last"] = v 

    def __output_info (self):
        """Outputs the stored info."""
        output = self._new_output()
        i = 0
        for sensor in self.__sensorlist:
            if sensor["enabled"]:
                output.set ("label[%i]" % i, sensor["label"])
                output.set ("value[%i]" % i, sensor["value"])

                if self.updated and "plot" in sensor:
                    # Only update the plotters when new values were read.
                    output.set ("plot[%i]" % i, sensor["plot"]);
                
                i = i + 1
        if i:
            output.set ("count", i);
        else:
            output.set ("label[0]", "No sensors.");
            output.set ("value[0]", "");
            output.set ("count", 1);
        self._send_output(output)
        self.updated = False
        
    def __tick (self):
        """Executed once a second inside the main thread."""
        if self.__active:
            # Check if the configuration was changed, and immediately re-read
            # sensors in that case.
            updated = False
            for sensor in self.__sensorlist:
                enabled = self._get_config(sensor["name"])
                if sensor["enabled"] != enabled:
                    if enabled: updated = True
                sensor["enabled"] = enabled
            if updated: self.__read_sensors ()
            # Update the display.
            self.__output_info ()
        else:
            output = self._new_output()
            output.set ("label[0]", "Reading sensors...");
            output.set ("value[0]", "");
            output.set ("count", 1);
            self._send_output(output)
        return True

    def __get_value_thread (self):
        """The work thread."""
        print "Activating lmsensors desklet..."
        self.__get_sensor_data (append = True)
        for sensor in self.__sensorlist:
            # whether to display it or not
            self._set_config_type (sensor["name"], TYPE_BOOL, 1);
            # 'enabled' doubles the same info, to detect config changes later
            sensor["enabled"] = self._get_config (sensor["name"])
        self.__active = 1
        while (not self._is_stopped()):
            time.sleep (self._get_config ("update"))
            self.__read_sensors ()

def new_sensor (args): return apply (LMSensorsSensor, args)
